<?php

/**
 * Webkul Software.
 *
 * @category  Webkul
 *
 * @author    Webkul Software Private Limited
 * @copyright Copyright (c) 2010-2017 Webkul Software Private Limited (https://webkul.com)
 * @license   https://store.webkul.com/license.html
 */

namespace Webkul\SalesforceConnector\Controller\Adminhtml\Categories;

class GetAllExportCategory extends \Magento\Backend\App\Action
{
    /** @var \Magento\Framework\View\Result\PageFactory */
    protected $_resultPageFactory = false;

    /** @var \Magento\Framework\Json\Helper\Data */
    protected $_json;
    protected $_helperData;

    /** @var \Webkul\Eshopsync\Model\CategoryFactory */
    protected $_categoryModel;

    /** @var \Magento\Catalog\Model\CategoryFactory */
    protected $_catalogCategoryModel;

    /**
     * @param \Magento\Backend\App\Action\Context
     * @param \Webkul\Eshopsync\Helper\Connection        $connection
     * @param \Magento\Framework\Json\Helper\Data        $jsonHelper
     * @param \Magento\Catalog\Model\CategoryFactory     $catalogCategory
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\Json\Helper\Data $jsonHelper,
        \Webkul\SalesforceConnector\Model\CategoryFactory $category,
        \Magento\Catalog\Model\CategoryFactory $catalogCategory,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Webkul\SalesforceConnector\Helper\Data $data
    ) {
        parent::__construct($context);
        $this->_helperData = $data;
        $this->_resultPageFactory = $resultPageFactory;
        $this->_json = $jsonHelper;
        $this->_categoryModel = $category;
        $this->_catalogCategoryModel = $catalogCategory;
    }

    public function execute()
    {
        $exportIds = [];
        if (\Webkul\SalesforceConnector\Helper\Connection::checkConnectionStatus()) {
            $syncEnable = $this->_helperData->getConfigValue('catPdt', 'synCat');
            if ($syncEnable) {
                $categoryModel = $this->_categoryModel->create();
                $catalogCategoryModel = $this->_catalogCategoryModel->create();
                $mappingCollection = $categoryModel
                    ->getCollection()
                    ->addFieldToSelect('magento_id')
                    ->addFieldToFilter('sync_status', '1')
                    ->getData();
                $mappedCategories = array_column($mappingCollection, 'magento_id');
                // foreach ($mappingCollection as $mapping) {
                //     array_push($mappedCategories, $mapping['magento_id']);
                // }

                if ($mappedCategories) {
                    $categoryCollection = $catalogCategoryModel->getCollection()
                        ->addAttributeToSelect('entity_id')
                        ->addAttributeToFilter('level', ['gt' => 0])
                        ->addAttributeToFilter('entity_id', ['nin' => $mappedCategories]);
                } else {
                    $categoryCollection = $catalogCategoryModel->getCollection()
                        ->addAttributeToSelect('entity_id')
                        ->addAttributeToFilter('level', ['gt' => 0]);
                }
                $exportIds = $categoryCollection->getAllIds();
                if (!$categoryCollection->getSize()) {
                    $this->messageManager->addNotice(__('All Magento Categories are already exported at salesforce.'));
                }
            } else {
                $this->messageManager->addError(__('Check Configuration'));
            }
        } else {
            $this->messageManager->addError(__('Check Connection'));
        }
        $response = ['categ_ids' => $exportIds];
        $this->getResponse()->setHeader('Content-type', 'application/json');
        $this->getResponse()->setBody($this->_json->jsonEncode($response));
    }

    /*
     * Check permission via ACL resource
     */
    // protected function _isAllowed()
    // {
    //     return true;
    // }
}
